@@ -14,7 +14,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Cons
--- Copyright   :  (C) 2012-15 Edward Kmett
+-- Copyright   :  (C) 2012-16 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -52,8 +52,6 @@ import Control.Lens.Tuple
 import Control.Lens.Type
 import qualified Data.ByteString      as StrictB
 import qualified Data.ByteString.Lazy as LazyB
-import           Data.List.NonEmpty   (NonEmpty(..))
-import qualified Data.List.NonEmpty   as NonEmpty
 import           Data.Monoid
 import qualified Data.Sequence as Seq
 import           Data.Sequence hiding ((<|), (|>), (:<), (:>))
@@ -121,12 +119,6 @@ instance Cons [a] [b] a b where
     []     -> Left  []
   {-# INLINE _Cons #-}
 
-instance a~b => Cons (NonEmpty a) (NonEmpty b) a b where
-  _Cons = prism' (uncurry NonEmpty.cons) $ \ xyz -> case xyz of
-    (x:|y:z) -> Just (x,y:|z)
-    _        -> Nothing
-  {-# INLINE _Cons #-}
-
 instance Cons (Seq a) (Seq b) a b where
   _Cons = prism (uncurry (Seq.<|)) $ \aas -> case viewl aas of
     a Seq.:< as -> Right (a, as)
@@ -352,12 +344,6 @@ instance Snoc [a] [b] a b where
     else Right (Prelude.init aas, Prelude.last aas)
   {-# INLINE _Snoc #-}
 
-instance a~b => Snoc (NonEmpty a) (NonEmpty b) a b where
-  _Snoc = prism' (\(x:|y,z) -> x:|y++[z]) $ \xyz -> case xyz of
-    x:|y
-      | Prelude.null y -> Nothing
-      | otherwise      -> Just (x :| Prelude.init y, Prelude.last y)
-
 instance Snoc (Seq a) (Seq b) a b where
   _Snoc = prism (uncurry (Seq.|>)) $ \aas -> case viewr aas of
     as Seq.:> a -> Right (as, a)
