@@ -20,7 +20,7 @@
 ----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Fold
--- Copyright   :  (C) 2012-15 Edward Kmett
+-- Copyright   :  (C) 2012-16 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -101,6 +101,7 @@ module Control.Lens.Fold
   , foldr1Of, foldl1Of
   , foldr1Of', foldl1Of'
   , foldrMOf, foldlMOf
+  , lookupOf
 
   -- * Indexed Folds
   , (^@..)
@@ -199,6 +200,7 @@ import Prelude hiding (foldr)
 {-# ANN module "HLint: ignore Eta reduce" #-}
 {-# ANN module "HLint: ignore Use camelCase" #-}
 {-# ANN module "HLint: ignore Use curry" #-}
+{-# ANN module "HLint: ignore Use fmap" #-}
 #endif
 
 infixl 8 ^.., ^?, ^?!, ^@.., ^@?, ^@?!
@@ -689,7 +691,7 @@ orOf l = getAny #. foldMapOf l Any
 -- @
 --
 -- @
--- 'ianyOf' l ≡ 'allOf' l '.' 'Indexed'
+-- 'ianyOf' l ≡ 'anyOf' l '.' 'Indexed'
 -- @
 --
 -- @
@@ -1296,7 +1298,7 @@ nullOf = hasn't
 -- Note: 'notNullOf' on a valid 'Iso', 'Lens' or 'Getter' should always return 'True'.
 --
 -- @
--- 'null' ≡ 'notNullOf' 'folded'
+-- 'not' '.' 'null' ≡ 'notNullOf' 'folded'
 -- @
 --
 -- This may be rather inefficient compared to the @'not' '.' 'null'@ check of many containers.
@@ -1518,6 +1520,24 @@ findMOf :: Monad m => Getting (Endo (m (Maybe a))) s a -> (a -> m Bool) -> s ->
 findMOf l f = foldrOf l (\a y -> f a >>= \r -> if r then return (Just a) else y) $ return Nothing
 {-# INLINE findMOf #-}
 
+-- | The 'lookupOf' function takes a 'Fold' (or 'Getter', 'Traversal',
+-- 'Lens', 'Iso', etc.), a key, and a structure containing key/value pairs.
+-- It returns the first value corresponding to the given key. This function
+-- generalizes 'lookup' to work on an arbitrary 'Fold' instead of lists.
+--
+-- >>> lookupOf folded 4 [(2, 'a'), (4, 'b'), (4, 'c')]
+-- Just 'b'
+--
+-- >>> lookupOf each 2 [(2, 'a'), (4, 'b'), (4, 'c')]
+-- Just 'a'
+--
+-- @
+-- 'lookupOf' :: 'Eq' k => 'Fold' s (k,v) -> k -> s -> 'Maybe' v
+-- @
+lookupOf :: Eq k => Getting (Endo (Maybe v)) s (k,v) -> k -> s -> Maybe v
+lookupOf l k = foldrOf l (\(k',v) next -> if k == k' then Just v else next) Nothing
+{-# INLINE lookupOf #-}
+
 -- | A variant of 'foldrOf' that has no base case and thus may only be applied
 -- to lenses and structures such that the 'Lens' views at least one element of
 -- the structure.
