@@ -4,10 +4,13 @@
 {-# LANGUAGE Trustworthy #-}
 #endif
 
+#ifndef MIN_VERSION_template_haskell
+#define MIN_VERSION_template_haskell(x,y,z) 1
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.FieldTH
--- Copyright   :  (C) 2014-2015 Edward Kmett, (C) 2014 Eric Mertens
+-- Copyright   :  (C) 2014-2016 Edward Kmett, (C) 2014 Eric Mertens
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -65,6 +68,16 @@ makeFieldOptics rules tyName =
 
 makeFieldOpticsForDec :: LensRules -> Dec -> DecsQ
 makeFieldOpticsForDec rules dec = case dec of
+#if MIN_VERSION_template_haskell(2,11,0)
+  DataD    _ tyName vars _ cons _ ->
+    makeFieldOpticsForDec' rules tyName (mkS tyName vars) cons
+  NewtypeD _ tyName vars _ con  _ ->
+    makeFieldOpticsForDec' rules tyName (mkS tyName vars) [con]
+  DataInstD _ tyName args _ cons _ ->
+    makeFieldOpticsForDec' rules tyName (tyName `conAppsT` args) cons
+  NewtypeInstD _ tyName args _ con _ ->
+    makeFieldOpticsForDec' rules tyName (tyName `conAppsT` args) [con]
+#else
   DataD    _ tyName vars cons _ ->
     makeFieldOpticsForDec' rules tyName (mkS tyName vars) cons
   NewtypeD _ tyName vars con  _ ->
@@ -73,6 +86,7 @@ makeFieldOpticsForDec rules dec = case dec of
     makeFieldOpticsForDec' rules tyName (tyName `conAppsT` args) cons
   NewtypeInstD _ tyName args con _ ->
     makeFieldOpticsForDec' rules tyName (tyName `conAppsT` args) [con]
+#endif
   _ -> fail "makeFieldOptics: Expected data or newtype type-constructor"
   where
   mkS tyName vars = tyName `conAppsT` map VarT (toListOf typeVars vars)
@@ -108,6 +122,8 @@ makeFieldOpticsForDec' rules tyName s cons =
 -- | Normalized the Con type into a uniform positional representation,
 -- eliminating the variance between records, infix constructors, and normal
 -- constructors.
+--
+-- For 'GadtC' and 'RecGadtC', the leftmost name is chosen.
 normalizeConstructor ::
   Con ->
   Q (Name, [(Maybe Name, Type)]) -- ^ constructor name, field name, field type
@@ -125,7 +141,13 @@ normalizeConstructor (ForallC _ _ con) =
   do con' <- normalizeConstructor con
      return (set (_2 . mapped . _1) Nothing con')
 
+#if MIN_VERSION_template_haskell(2,11,0)
+normalizeConstructor (GadtC ns xs _) =
+  return (head ns, [ (Nothing, ty) | (_,ty) <- xs])
 
+normalizeConstructor (RecGadtC ns xs _) =
+  return (head ns, [ (Just fieldName, ty) | (fieldName,_,ty) <- xs])
+#endif
 
 data OpticType = GetterType | LensType | IsoType
 
@@ -397,7 +419,7 @@ makeFieldClauses rules opticType cons =
 -- constructor.
 makePureClause :: Name -> Int -> ClauseQ
 makePureClause conName fieldCount =
-  do xs <- replicateM fieldCount (newName "x")
+  do xs <- newNames "x" fieldCount
      -- clause: _ (Con x1..xn) = pure (Con x1..xn)
      clause [wildP, conP conName (map varP xs)]
             (normalB (appE (varE pureValName) (appsE (conE conName : map varE xs))))
@@ -410,7 +432,7 @@ makeGetterClause :: Name -> Int -> [Int] -> ClauseQ
 makeGetterClause conName fieldCount []     = makePureClause conName fieldCount
 makeGetterClause conName fieldCount fields =
   do f  <- newName "f"
-     xs <- replicateM (length fields) (newName "x")
+     xs <- newNames "x" (length fields)
 
      let pats (i:is) (y:ys)
            | i `elem` fields = varP y : pats is ys
@@ -435,8 +457,8 @@ makeFieldOpticClause conName fieldCount [] _ =
   makePureClause conName fieldCount
 makeFieldOpticClause conName fieldCount (field:fields) irref =
   do f  <- newName "f"
-     xs <- replicateM fieldCount          (newName "x")
-     ys <- replicateM (1 + length fields) (newName "y")
+     xs <- newNames "x" fieldCount
+     ys <- newNames "y" (1 + length fields)
 
      let xs' = foldr (\(i,x) -> set (ix i) x) xs (zip (field:fields) ys)
 
@@ -541,7 +563,7 @@ applyTypeSubst sub = rewrite aux
 -- Field generation parameters
 ------------------------------------------------------------------------
 
-
+-- | Rules to construct lenses for data fields.
 data LensRules = LensRules
   { _simpleLenses    :: Bool
   , _generateSigs    :: Bool
