@@ -17,7 +17,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.Indexed
--- Copyright   :  (C) 2012-2015 Edward Kmett
+-- Copyright   :  (C) 2012-2016 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -54,6 +54,7 @@ import Data.Distributive
 import Data.Functor.Bind
 import Data.Functor.Contravariant
 import Data.Int
+import Data.Profunctor.Closed
 import Data.Profunctor
 import Data.Profunctor.Rep
 import Data.Profunctor.Sieve
@@ -74,7 +75,7 @@ import Control.Lens.Internal.Coerce
 class
   ( Choice p, Corepresentable p, Comonad (Corep p), Traversable (Corep p)
   , Strong p, Representable p, Monad (Rep p), MonadFix (Rep p), Distributive (Rep p)
-  , Costrong p, ArrowLoop p, ArrowApply p, ArrowChoice p
+  , Costrong p, ArrowLoop p, ArrowApply p, ArrowChoice p, Closed p
   ) => Conjoined p where
 
   -- | 'Conjoined' is strong enough to let us distribute every 'Conjoined'
@@ -141,7 +142,7 @@ instance Bind (Indexed i a) where
   {-# INLINE (>>-) #-}
 
 instance Monad (Indexed i a) where
-  return b = Indexed $ \_ _ -> b
+  return = pure
   {-# INLINE return #-}
   Indexed f >>= k = Indexed $ \i a -> runIndexed (k (f i a)) i a
   {-# INLINE (>>=) #-}
@@ -164,6 +165,9 @@ instance Profunctor (Indexed i) where
   {-# INLINE ( #. ) #-}
 #endif
 
+instance Closed (Indexed i) where
+  closed (Indexed iab) = Indexed $ \i xa x -> iab i (xa x)
+
 instance Costrong (Indexed i) where
   unfirst (Indexed iadbd) = Indexed $ \i a -> let
       (b, d) = iadbd i (a, d)
