@@ -3,10 +3,13 @@
 {-# LANGUAGE Trustworthy #-}
 #endif
 
+#ifndef MIN_VERSION_template_haskell
+#define MIN_VERSION_template_haskell(x,y,z) 1
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Internal.PrismTH
--- Copyright   :  (C) 2014 Edward Kmett, (C) 2014 Eric Mertens
+-- Copyright   :  (C) 2014-2016 Edward Kmett and Eric Mertens
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -94,7 +97,7 @@ makePrisms = makePrisms' True
 --
 -- instance AsFooBarBaz (FooBarBaz a) a
 -- @
--- 
+--
 -- Generate an "As" class of prisms. Names are selected by prefixing the constructor
 -- name with an underscore.  Constructors with multiple fields will
 -- construct Prisms to tuples of those fields.
@@ -114,16 +117,23 @@ makePrisms' normal typeName =
 -- | Generate prisms for the given 'Dec'
 makeDecPrisms :: Bool {- ^ generate top-level definitions -} -> Dec -> DecsQ
 makeDecPrisms normal dec = case dec of
-  DataD        _ ty vars cons _ -> next ty (convertTVBs vars) cons
-  NewtypeD     _ ty vars con  _ -> next ty (convertTVBs vars) [con]
-  DataInstD    _ ty tys  cons _ -> next ty tys                cons
-  NewtypeInstD _ ty tys  con  _ -> next ty tys                [con]
+#if MIN_VERSION_template_haskell(2,11,0)
+  DataD        _ ty vars _ cons _ -> next ty (convertTVBs vars) cons
+  NewtypeD     _ ty vars _ con  _ -> next ty (convertTVBs vars) [con]
+  DataInstD    _ ty tys  _ cons _ -> next ty tys                cons
+  NewtypeInstD _ ty tys  _ con  _ -> next ty tys                [con]
+#else
+  DataD        _ ty vars   cons _ -> next ty (convertTVBs vars) cons
+  NewtypeD     _ ty vars   con  _ -> next ty (convertTVBs vars) [con]
+  DataInstD    _ ty tys    cons _ -> next ty tys                cons
+  NewtypeInstD _ ty tys    con  _ -> next ty tys                [con]
+#endif
   _                             -> fail "makePrisms: expected type constructor dec"
   where
   convertTVBs = map (VarT . bndrName)
 
   next ty args cons =
-    makeConsPrisms (conAppsT ty args) (map normalizeCon cons) cls
+    makeConsPrisms (conAppsT ty args) (concatMap normalizeCon cons) cls
     where
     cls | normal    = Nothing
         | otherwise = Just ty
@@ -321,7 +331,7 @@ makeReviewer conName fields =
 makeSimpleRemitter :: Name -> Int -> ExpQ
 makeSimpleRemitter conName fields =
   do x  <- newName "x"
-     xs <- replicateM fields (newName "y")
+     xs <- newNames "y" fields
      let matches =
            [ match (conP conName (map varP xs))
                    (normalB (appE (conE rightDataName) (toTupleE (map varE xs))))
@@ -343,7 +353,7 @@ makeFullRemitter cons target =
      lam1E (varP x) (caseE (varE x) (map mkMatch cons))
   where
   mkMatch (NCon conName _ n) =
-    do xs <- replicateM (length n) (newName "y")
+    do xs <- newNames "y" (length n)
        match (conP conName (map varP xs))
              (normalB
                (if conName == target
@@ -357,7 +367,7 @@ makeFullRemitter cons target =
 -- (\(Con x y z) -> (x,y,z)) :: s -> a
 makeIsoRemitter :: Name -> Int -> ExpQ
 makeIsoRemitter conName fields =
-  do xs <- replicateM fields (newName "x")
+  do xs <- newNames "x" fields
      lam1E (conP conName (map varP xs))
            (toTupleE (map varE xs))
 
@@ -463,17 +473,23 @@ nconTypes :: Lens' NCon [Type]
 nconTypes f x = fmap (\y -> x {_nconTypes = y}) (f (_nconTypes x))
 
 
--- | Normalize 'Con' to its constructor name and field types.
-normalizeCon :: Con -> NCon
-normalizeCon (RecC    conName xs) = NCon conName Nothing (map (view _3) xs)
-normalizeCon (NormalC conName xs) = NCon conName Nothing (map (view _2) xs)
-normalizeCon (InfixC (_,x) conName (_,y)) = NCon conName Nothing [x,y]
+-- | Normalize a single 'Con' to its constructor name and field types.
+-- Because GADT syntax allows multiple constructors to be defined at
+-- the same time, this function can return multiple normalized results.
+normalizeCon :: Con -> [NCon]
+normalizeCon (RecC    conName xs) = [NCon conName Nothing (map (view _3) xs)]
+normalizeCon (NormalC conName xs) = [NCon conName Nothing (map (view _2) xs)]
+normalizeCon (InfixC (_,x) conName (_,y)) = [NCon conName Nothing [x,y]]
 normalizeCon (ForallC [] [] con) = normalizeCon con -- happens in GADTs
-normalizeCon (ForallC _ cx con) = NCon n (cx1 <> cx2) tys
-  where
-  cx1 = Just cx
-  NCon n cx2 tys = normalizeCon con
-
+normalizeCon (ForallC _ cx1 con) =
+  [NCon n (Just cx1 <> cx2) tys
+     | NCon n cx2 tys <- normalizeCon con ]
+#if MIN_VERSION_template_haskell(2,11,0)
+normalizeCon (GadtC conNames xs _)    =
+  [ NCon conName Nothing (map (view _2) xs) | conName <- conNames ]
+normalizeCon (RecGadtC conNames xs _) =
+  [ NCon conName Nothing (map (view _3) xs) | conName <- conNames ]
+#endif
 
 -- | Compute a prism's name by prefixing an underscore for normal
 -- constructors and period for operators.
