@@ -14,7 +14,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.TH
--- Copyright   :  (C) 2012-15 Edward Kmett, 2012-13 Michael Sloan
+-- Copyright   :  (C) 2012-16 Edward Kmett, 2012-13 Michael Sloan
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -24,38 +24,57 @@
 module Control.Lens.TH
   (
   -- * Constructing Lenses Automatically
+  -- ** Lenses for data fields
     makeLenses, makeLensesFor
   , makeClassy, makeClassyFor, makeClassy_
+  , makeFields
+  -- ** Prisms
   , makePrisms
   , makeClassyPrisms
+  -- ** Wrapped
   , makeWrapped
-  , makeFields
   -- * Constructing Lenses Given a Declaration Quote
+  -- ** Lenses for data fields
   , declareLenses, declareLensesFor
   , declareClassy, declareClassyFor
-  , declarePrisms
-  , declareWrapped
   , declareFields
+  -- ** Prisms
+  , declarePrisms
+  -- ** Wrapped
+  , declareWrapped
   -- * Configuring Lenses
+  -- ** Running LensRules
   , makeLensesWith
   , declareLensesWith
-  , defaultFieldRules
-  , camelCaseFields
-  , underscoreFields
-  , abbreviatedFields
+  -- ** LensRules type
   , LensRules
-  , DefName(..)
+  -- ** Predefined LensRules
   , lensRules
   , lensRulesFor
   , classyRules
   , classyRules_
+  , defaultFieldRules
+  , camelCaseFields
+  , underscoreFields
+  , abbreviatedFields
+  -- ** LensRules configuration accessors
   , lensField
+  , FieldNamer
+  , DefName(..)
   , lensClass
+  , ClassyNamer
   , simpleLenses
   , createClass
   , generateSignatures
   , generateUpdateableOptics
   , generateLazyPatterns
+  -- ** FieldNamers
+  , underscoreNoPrefixNamer
+  , lookingupNamer
+  , mappingNamer
+  , camelCaseNamer
+  , underscoreNamer
+  , abbreviatedNamer
   ) where
 
 #if !MIN_VERSION_base(4,8,0)
@@ -75,6 +94,8 @@ import Control.Lens.Traversal
 import Control.Lens.Internal.TH
 import Control.Lens.Internal.FieldTH
 import Control.Lens.Internal.PrismTH
+import Control.Lens.Wrapped () -- haddocks
+import Control.Lens.Type () -- haddocks
 import Data.Char (toLower, isUpper)
 import Data.Foldable hiding (concat, any)
 import Data.List as List
@@ -86,6 +107,7 @@ import Data.Set.Lens
 import Data.Traversable hiding (mapM)
 import Language.Haskell.TH
 import Language.Haskell.TH.Lens
+import Language.Haskell.TH.Syntax
 
 #ifdef HLINT
 {-# ANN module "HLint: ignore Eta reduce" #-}
@@ -129,7 +151,7 @@ generateUpdateableOptics f r =
 -- > undefined & x .~ 8 & y .~ True
 -- Foo {_x = 8, _y = True}
 -- @
--- 
+--
 -- The downside of this flag is that it can lead to space-leaks and
 -- code-size/compile-time increases when generated for large records. By
 -- default this flag is turned off, and strict optics are generated.
@@ -151,30 +173,31 @@ createClass f r =
   fmap (\x -> r { _generateClasses = x}) (f (_generateClasses r))
 
 -- | 'Lens'' to access the convention for naming fields in our 'LensRules'.
---
--- Defaults to stripping the _ off of the field name, lowercasing the name, and
--- skipping the field if it doesn't start with an '_'. The field naming rule
--- provides the names of all fields in the type as well as the current field.
--- This extra generality enables field naming conventions that depend on the
--- full set of names in a type.
---
--- The field naming rule has access to the type name, the names of all the field
--- of that type (including the field being named), and the name of the field
--- being named.
---
--- TypeName -> FieldNames -> FieldName -> DefinitionNames
-lensField :: Lens' LensRules (Name -> [Name] -> Name -> [DefName])
+lensField :: Lens' LensRules FieldNamer
 lensField f r = fmap (\x -> r { _fieldToDef = x}) (f (_fieldToDef r))
 
--- | Retrieve options such as the name of the class and method to put in it to
--- build a class around monomorphic data types. "Classy" lenses are generated
--- when this naming convention is provided.
--- TypeName -> Maybe (ClassName, MainMethodName)
-lensClass :: Lens' LensRules (Name -> Maybe (Name, Name))
+-- | The rule to create function names of lenses for data fields.
+--
+-- Although it's sometimes useful, you won't need the first two
+-- arguments most of the time.
+type FieldNamer = Name -- ^ Name of the data type that lenses are being generated for.
+                  -> [Name] -- ^ Names of all fields (including the field being named) in the data type.
+                  -> Name -- ^ Name of the field being named.
+                  -> [DefName] -- ^ Name(s) of the lens functions. If empty, no lens is created for that field.
+
+-- | 'Lens'' to access the option for naming "classy" lenses.
+lensClass :: Lens' LensRules ClassyNamer
 lensClass f r = fmap (\x -> r { _classyLenses = x }) (f (_classyLenses r))
 
+-- | The optional rule to create a class and method around a
+-- monomorphic data type. If this naming convention is provided, it
+-- generates a "classy" lens.
+type ClassyNamer = Name -- ^ Name of the data type that lenses are being generated for.
+                   -> Maybe (Name, Name) -- ^ Names of the class and the main method it generates, respectively.
+
 -- | Rules for making fairly simple partial lenses, ignoring the special cases
 -- for isomorphisms and traversals, and not making any classes.
+-- It uses 'underscoreNoPrefixNamer'.
 lensRules :: LensRules
 lensRules = LensRules
   { _simpleLenses    = False
@@ -184,23 +207,37 @@ lensRules = LensRules
   , _allowUpdates    = True
   , _lazyPatterns    = False
   , _classyLenses    = const Nothing
-  , _fieldToDef      = \_ _ n ->
-       case nameBase n of
-         '_':x:xs -> [TopName (mkName (toLower x:xs))]
-         _        -> []
+  , _fieldToDef      = underscoreNoPrefixNamer
   }
 
+-- | A 'FieldNamer' that strips the _ off of the field name,
+-- lowercases the name, and skips the field if it doesn't start with
+-- an '_'.
+underscoreNoPrefixNamer :: FieldNamer
+underscoreNoPrefixNamer _ _ n =
+  case nameBase n of
+    '_':x:xs -> [TopName (mkName (toLower x:xs))]
+    _        -> []
+
+
 -- | Construct a 'LensRules' value for generating top-level definitions
 -- using the given map from field names to definition names.
 lensRulesFor ::
   [(String, String)] {- ^ [(Field Name, Definition Name)] -} ->
   LensRules
-lensRulesFor fields = lensRules & lensField .~ mkNameLookup fields
+lensRulesFor fields = lensRules & lensField .~ lookingupNamer fields
 
-mkNameLookup :: [(String,String)] -> Name -> [Name] -> Name -> [DefName]
-mkNameLookup kvs _ _ field =
+-- | Create a 'FieldNamer' from explicit pairings of @(fieldName, lensName)@.
+lookingupNamer :: [(String,String)] -> FieldNamer
+lookingupNamer kvs _ _ field =
   [ TopName (mkName v) | (k,v) <- kvs, k == nameBase field]
 
+-- | Create a 'FieldNamer' from a mapping function. If the function
+-- returns @[]@, it creates no lens for the field.
+mappingNamer :: (String -> [String]) -- ^ A function that maps a @fieldName@ to @lensName@s.
+             -> FieldNamer
+mappingNamer mapper _ _ = fmap (TopName . mkName) . mapper . nameBase
+
 -- | Rules for making lenses and traversals that precompose another 'Lens'.
 classyRules :: LensRules
 classyRules = LensRules
@@ -214,10 +251,7 @@ classyRules = LensRules
         case nameBase n of
           x:xs -> Just (mkName ("Has" ++ x:xs), mkName (toLower x:xs))
           []   -> Nothing
-  , _fieldToDef      = \_ _ n ->
-        case nameBase n of
-          '_':x:xs -> [TopName (mkName (toLower x:xs))]
-          _        -> []
+  , _fieldToDef      = underscoreNoPrefixNamer
   }
 
 -- | Rules for making lenses and traversals that precompose another 'Lens'
@@ -229,8 +263,9 @@ classyRulesFor
   LensRules
 classyRulesFor classFun fields = classyRules
   & lensClass .~ (over (mapped . both) mkName . classFun . nameBase)
-  & lensField .~ mkNameLookup fields
+  & lensField .~ lookingupNamer fields
 
+-- | A 'LensRules' used by 'makeClassy_'.
 classyRules_ :: LensRules
 classyRules_
   = classyRules & lensField .~ \_ _ n -> [TopName (mkName ('_':nameBase n))]
@@ -400,7 +435,7 @@ declareClassyFor classes fields
   $ classyRulesFor (`Prelude.lookup`classes) fields
   & lensField .~ \_ _ n -> [TopName n]
 
--- | Generate a 'Prism' for each constructor of each data type.
+-- | Generate a 'Control.Lens.Type.Prism' for each constructor of each data type.
 --
 -- /e.g./
 --
@@ -423,7 +458,7 @@ declarePrisms = declareWith $ \dec -> do
   emit =<< Trans.lift (makeDecPrisms True dec)
   return dec
 
--- | Build 'Wrapped' instance for each newtype.
+-- | Build 'Control.Lens.Wrapped.Wrapped' instance for each newtype.
 declareWrapped :: DecsQ -> DecsQ
 declareWrapped = declareWith $ \dec -> do
   maybeDecs <- Trans.lift (makeWrappedForDec dec)
@@ -449,8 +484,13 @@ declareLensesWith rules = declareWith $ \dec -> do
 -- | Transform @NewtypeD@s declarations to @DataD@s and @NewtypeInstD@s to
 -- @DataInstD@s.
 deNewtype :: Dec -> Dec
+#if MIN_VERSION_template_haskell(2,11,0)
+deNewtype (NewtypeD ctx tyName args kind c d) = DataD ctx tyName args kind [c] d
+deNewtype (NewtypeInstD ctx tyName args kind c d) = DataInstD ctx tyName args kind [c] d
+#else
 deNewtype (NewtypeD ctx tyName args c d) = DataD ctx tyName args [c] d
 deNewtype (NewtypeInstD ctx tyName args c d) = DataInstD ctx tyName args [c] d
+#endif
 deNewtype d = d
 
 
@@ -466,14 +506,22 @@ apps = Prelude.foldl AppT
 
 makeDataDecl :: Dec -> Maybe DataDecl
 makeDataDecl dec = case deNewtype dec of
-  DataD ctx tyName args cons _ -> Just DataDecl
+  DataD ctx tyName args
+#if MIN_VERSION_template_haskell(2,11,0)
+        _
+#endif
+        cons _ -> Just DataDecl
     { dataContext = ctx
     , tyConName = Just tyName
     , dataParameters = args
     , fullType = apps $ ConT tyName
     , constructors = cons
     }
-  DataInstD ctx familyName args cons _ -> Just DataDecl
+  DataInstD ctx familyName args
+#if MIN_VERSION_template_haskell(2,11,0)
+            _
+#endif
+            cons _ -> Just DataDecl
     { dataContext = ctx
     , tyConName = Nothing
     , dataParameters = map PlainTV vars
@@ -597,7 +645,8 @@ overHead f (x:xs) = f x : xs
 underscoreFields :: LensRules
 underscoreFields = defaultFieldRules & lensField .~ underscoreNamer
 
-underscoreNamer :: Name -> [Name] -> Name -> [DefName]
+-- | A 'FieldNamer' for 'underscoreFields'.
+underscoreNamer :: FieldNamer
 underscoreNamer _ _ field = maybeToList $ do
   _      <- prefix field'
   method <- niceLens
@@ -620,7 +669,8 @@ underscoreNamer _ _ field = maybeToList $ do
 camelCaseFields :: LensRules
 camelCaseFields = defaultFieldRules
 
-camelCaseNamer :: Name -> [Name] -> Name -> [DefName]
+-- | A 'FieldNamer' for 'camelCaseFields'.
+camelCaseNamer :: FieldNamer
 camelCaseNamer tyName fields field = maybeToList $ do
 
   fieldPart <- stripPrefix expectedPrefix (nameBase field)
@@ -648,7 +698,8 @@ camelCaseNamer tyName fields field = maybeToList $ do
 abbreviatedFields :: LensRules
 abbreviatedFields = defaultFieldRules { _fieldToDef = abbreviatedNamer }
 
-abbreviatedNamer :: Name -> [Name] -> Name -> [DefName]
+-- | A 'FieldNamer' for 'abbreviatedFields'.
+abbreviatedNamer :: FieldNamer
 abbreviatedNamer _ fields field = maybeToList $ do
 
   fieldPart <- stripMaxLc (nameBase field)
@@ -746,12 +797,25 @@ traverseDataAndNewtype f decs = traverse go decs
 
       -- Recurse into instance declarations because they main contain
       -- associated data family instances.
-      InstanceD ctx inst body -> InstanceD ctx inst <$> traverse go body
-
+#if MIN_VERSION_template_haskell(2,11,0)
+      InstanceD moverlap ctx inst body -> InstanceD moverlap ctx inst <$> traverse go body
+#else
+      InstanceD ctx inst body -> InstanceD  ctx inst <$> traverse go body
+#endif
       _ -> pure dec
 
 stripFields :: Dec -> Dec
 stripFields dec = case dec of
+#if MIN_VERSION_template_haskell(2,11,0)
+  DataD ctx tyName tyArgs kind cons derivings ->
+    DataD ctx tyName tyArgs kind (map deRecord cons) derivings
+  NewtypeD ctx tyName tyArgs kind con derivings ->
+    NewtypeD ctx tyName tyArgs kind (deRecord con) derivings
+  DataInstD ctx tyName tyArgs kind cons derivings ->
+    DataInstD ctx tyName tyArgs kind (map deRecord cons) derivings
+  NewtypeInstD ctx tyName tyArgs kind con derivings ->
+    NewtypeInstD ctx tyName tyArgs kind (deRecord con) derivings
+#else
   DataD ctx tyName tyArgs cons derivings ->
     DataD ctx tyName tyArgs (map deRecord cons) derivings
   NewtypeD ctx tyName tyArgs con derivings ->
@@ -760,6 +824,7 @@ stripFields dec = case dec of
     DataInstD ctx tyName tyArgs (map deRecord cons) derivings
   NewtypeInstD ctx tyName tyArgs con derivings ->
     NewtypeInstD ctx tyName tyArgs (deRecord con) derivings
+#endif
   _ -> dec
 
 deRecord :: Con -> Con
@@ -767,4 +832,14 @@ deRecord con@NormalC{} = con
 deRecord con@InfixC{} = con
 deRecord (ForallC tyVars ctx con) = ForallC tyVars ctx $ deRecord con
 deRecord (RecC conName fields) = NormalC conName (map dropFieldName fields)
-  where dropFieldName (_, str, typ) = (str, typ)
+#if MIN_VERSION_template_haskell(2,11,0)
+deRecord con@GadtC{} = con
+deRecord (RecGadtC ns fields retTy) = GadtC ns (map dropFieldName fields) retTy
+#endif
+
+#if MIN_VERSION_template_haskell(2,11,0)
+dropFieldName :: VarBangType   -> BangType
+#else
+dropFieldName :: VarStrictType -> StrictType
+#endif
+dropFieldName (_, str, typ) = (str, typ)
