@@ -14,7 +14,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Traversal
--- Copyright   :  (C) 2012-15 Edward Kmett
+-- Copyright   :  (C) 2012-16 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -34,12 +34,12 @@
 -- type 'Traversal' s t a b = forall f. 'Applicative' f => (a -> f b) -> s -> f t
 -- @
 --
--- While a 'Traversal' isn't quite a 'Fold', it _can_ be used for
--- 'Control.Lens.Getter.Getting' like a 'Fold', because given a
--- 'Data.Monoid.Monoid' @m@, we have an 'Applicative'
--- for @('Const' m)@. Everything you know how to do with a 'Traversable'
--- container, you can with with a 'Traversal', and here we provide
--- combinators that generalize the usual 'Traversable' operations.
+-- A 'Traversal' can be used as a 'Fold'.
+-- Any 'Traversal' can be used for 'Control.Lens.Getter.Getting' like a 'Fold',
+-- because given a 'Data.Monoid.Monoid' @m@, we have an 'Applicative' for
+-- @('Const' m)@. Everything you know how to do with a 'Traversable' container,
+-- you can with a 'Traversal', and here we provide combinators that generalize
+-- the usual 'Traversable' operations.
 ----------------------------------------------------------------------------
 module Control.Lens.Traversal
   (
@@ -148,7 +148,7 @@ import Data.Bitraversable
 import Data.Foldable (Foldable)
 #endif
 import Data.Functor.Compose
-import Data.Functor.Kan.Rift
+import Data.Functor.Day.Curried
 import Data.Functor.Yoneda
 import Data.Int
 import Data.IntMap as IntMap
@@ -1276,7 +1276,7 @@ deepOf r l pafb = go
 -- However, @foo@ and @bar@ are each going to use the 'Applicative' they are given.
 --
 -- 'confusing' exploits the 'Yoneda' lemma to merge their separate uses of 'fmap' into a single 'fmap'.
--- and it further exploits an interesting property of the right Kan lift (or 'Rift') to left associate
+-- and it further exploits an interesting property of the right Kan lift (or 'Curried') to left associate
 -- all of the uses of '(<*>)' to make it possible to fuse together more fmaps.
 --
 -- This is particularly effective when the choice of functor 'f' is unknown at compile
@@ -1288,12 +1288,12 @@ deepOf r l pafb = go
 -- @
 -- 'confusing' :: 'Traversal' s t a b -> 'Traversal' s t a b
 -- @
-confusing :: Applicative f => LensLike (Rift (Yoneda f) (Yoneda f)) s t a b -> LensLike f s t a b
-confusing t = \f -> lowerYoneda . lowerRift . t (liftRiftYoneda . f)
+confusing :: Applicative f => LensLike (Curried (Yoneda f) (Yoneda f)) s t a b -> LensLike f s t a b
+confusing t = \f -> lowerYoneda . lowerCurried . t (liftCurriedYoneda . f)
   where
-  liftRiftYoneda :: Applicative f => f a -> Rift (Yoneda f) (Yoneda f) a
-  liftRiftYoneda fa = Rift (`yap` fa)
-  {-# INLINE liftRiftYoneda #-}
+  liftCurriedYoneda :: Applicative f => f a -> Curried (Yoneda f) (Yoneda f) a
+  liftCurriedYoneda fa = Curried (`yap` fa)
+  {-# INLINE liftCurriedYoneda #-}
 
   yap :: Applicative f => Yoneda f (a -> b) -> f a -> Yoneda f b
   yap (Yoneda k) fa = Yoneda (\ab_r -> k (ab_r .) <*> fa)
