@@ -29,7 +29,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Lens.Wrapped
--- Copyright   :  (C) 2012-15 Edward Kmett, Michael Sloan
+-- Copyright   :  (C) 2012-16 Edward Kmett, Michael Sloan
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -105,7 +105,6 @@ import           Data.Functor.Compose
 import           Data.Functor.Contravariant
 import qualified Data.Functor.Contravariant.Compose as Contravariant
 import           Data.Functor.Constant
-import           Data.Functor.Coproduct
 import           Data.Functor.Identity
 import           Data.Functor.Reverse
 import           Data.Hashable
@@ -277,7 +276,7 @@ instance Wrapped (Alt f a) where
   {-# INLINE _Wrapped' #-}
 #endif
 
-instance (t ~ ArrowMonad m' a', ArrowApply m) => Rewrapped (ArrowMonad m a) t
+instance t ~ ArrowMonad m' a' => Rewrapped (ArrowMonad m a) t
 instance Wrapped (ArrowMonad m a) where
   type Unwrapped (ArrowMonad m a) = m () a
   _Wrapped' = iso getArrowMonad ArrowMonad
@@ -390,12 +389,6 @@ instance Wrapped (Strict.WriterT w m a) where
 
 -- * comonad-transformers
 
-instance (t ~ Coproduct f' g' a') => Rewrapped (Coproduct f g a) t
-instance Wrapped (Coproduct f g a) where
-  type Unwrapped (Coproduct f g a) = Either (f a) (g a)
-  _Wrapped' = iso getCoproduct Coproduct
-  {-# INLINE _Wrapped' #-}
-
 instance (t ~ TracedT m' w' a') => Rewrapped (TracedT m w a) t
 instance Wrapped (TracedT m w a) where
   type Unwrapped (TracedT m w a) = w (m -> a)
@@ -615,7 +608,11 @@ instance Wrapped ErrorCall where
   {-# INLINE _Wrapped' #-}
 
 getErrorCall :: ErrorCall -> String
+#if __GLASGOW_HASKELL__ < 800
 getErrorCall (ErrorCall x) = x
+#else
+getErrorCall (ErrorCallWithLocation x _) = x
+#endif
 {-# INLINE getErrorCall #-}
 
 getRecUpdError :: RecUpdError -> String
