@@ -1,10 +1,11 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE ExistentialQuantification #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE FlexibleInstances #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Control.Monad.Error.Lens
--- Copyright   :  (C) 2014-2015 Edward Kmett
+-- Copyright   :  (C) 2014-2016 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  provisional
@@ -37,6 +38,10 @@ import qualified Data.Monoid as M
 import Data.Semigroup (Semigroup(..))
 import Prelude
 
+#ifdef HLINT
+{-# ANN module "HLint: ignore Use fmap" #-}
+#endif
+
 ------------------------------------------------------------------------------
 -- Catching
 ------------------------------------------------------------------------------
