@@ -10,7 +10,7 @@
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.Haskell.TH.Lens
--- Copyright   :  (C) 2012-2015 Edward Kmett
+-- Copyright   :  (C) 2012-2016 Edward Kmett
 -- License     :  BSD-style (see the file LICENSE)
 -- Maintainer  :  Edward Kmett <ekmett@gmail.com>
 -- Stability   :  experimental
@@ -60,6 +60,19 @@ module Language.Haskell.TH.Lens
   -- ** TySynEqn Lenses
   , tySynEqnPatterns
   , tySynEqnResult
+#endif
+#if MIN_VERSION_template_haskell(2,11,0)
+  -- ** InjectivityAnn Lenses
+  , injectivityAnnOutput
+  , injectivityAnnInputs
+  -- ** TypeFamilyHead Lenses
+  , typeFamilyHeadName
+  , typeFamilyHeadTyVarBndrs
+  , typeFamilyHeadResultSig
+  , typeFamilyHeadInjectivityAnn
+  -- ** Bang Prisms
+  , bangSourceUnpackedness
+  , bangSourceStrictness
 #endif
   -- * Prisms
   -- ** Info Prisms
@@ -85,7 +98,6 @@ module Language.Haskell.TH.Lens
   , _InfixD
 #endif
   , _PragmaD
-  , _FamilyD
   , _DataInstD
   , _NewtypeInstD
   , _TySynInstD
@@ -96,16 +108,48 @@ module Language.Haskell.TH.Lens
 #if MIN_VERSION_template_haskell(2,10,0)
   , _StandaloneDerivD
   , _DefaultSigD
+#endif
+#if MIN_VERSION_template_haskell(2,11,0)
+  , _DataFamilyD
+  , _OpenTypeFamilyD
+#else
+  , _FamilyD
 #endif
   -- ** Con Prisms
   , _NormalC
   , _RecC
   , _InfixC
   , _ForallC
+#if MIN_VERSION_template_haskell(2,11,0)
+  , _GadtC
+  , _RecGadtC
+#endif
+#if MIN_VERSION_template_haskell(2,11,0)
+  -- ** Overlap Prisms
+  ,_Overlappable
+  ,_Overlapping
+  ,_Overlaps
+  ,_Incoherent
+#endif
+#if MIN_VERSION_template_haskell(2,11,0)
+  -- ** SourceUnpackedness Prisms
+  , _NoSourceUnpackedness
+  , _SourceNoUnpack
+  , _SourceUnpack
+  -- ** SourceStrictness Prisms
+  , _NoSourceStrictness
+  , _SourceLazy
+  , _SourceStrict
+  -- ** DecidedStrictness Prisms
+  , _DecidedLazy
+  , _DecidedStrict
+  , _DecidedUnpack
+#else
   -- ** Strict Prisms
   , _IsStrict
   , _NotStrict
   , _Unpacked
+#endif
   -- ** Foreign Prisms
   , _ImportF
   , _ExportF
@@ -192,6 +236,9 @@ module Language.Haskell.TH.Lens
   , _RecUpdE
 #if MIN_VERSION_template_haskell(2,10,0)
   , _StaticE
+#endif
+#if MIN_VERSION_template_haskell(2,11,0)
+  , _UnboundVarE
 #endif
   -- ** Body Prisms
   , _GuardedB
@@ -219,6 +266,9 @@ module Language.Haskell.TH.Lens
   , _FloatPrimL
   , _DoublePrimL
   , _StringPrimL
+#if MIN_VERSION_template_haskell(2,11,0)
+  , _CharPrimL
+#endif
   -- ** Pat Prisms
   , _LitP
   , _VarP
@@ -259,10 +309,22 @@ module Language.Haskell.TH.Lens
   , _StarT
   , _ConstraintT
   , _LitT
+#endif
+#if MIN_VERSION_template_haskell(2,11,0)
+  , _InfixT
+  , _UInfixT
+  , _ParensT
+  , _WildCardT
 #endif
   -- ** TyVarBndr Prisms
   , _PlainTV
   , _KindedTV
+#if MIN_VERSION_template_haskell(2,11,0)
+  -- ** FamilyResultSig Prisms
+  , _NoSig
+  , _KindSig
+  , _TyVarSig
+#endif
 #if MIN_VERSION_template_haskell(2,8,0)
   -- ** TyLit Prisms
   , _NumTyLit
@@ -316,11 +378,40 @@ instance HasName TyVarBndr where
 instance HasName Name where
   name = id
 
+-- | On @template-haskell-2.11.0.0@ or later, if a 'GadtC' or 'RecGadtC' has
+-- multiple 'Name's, the leftmost 'Name' will be chosen.
 instance HasName Con where
   name f (NormalC n tys)       = (`NormalC` tys) <$> f n
   name f (RecC n tys)          = (`RecC` tys) <$> f n
   name f (InfixC l n r)        = (\n' -> InfixC l n' r) <$> f n
   name f (ForallC bds ctx con) = ForallC bds ctx <$> name f con
+#if MIN_VERSION_template_haskell(2,11,0)
+  name f (GadtC ns argTys retTy) =
+    (\n -> GadtC [n] argTys retTy) <$> f (head ns)
+  name f (RecGadtC ns argTys retTy) =
+    (\n -> RecGadtC [n] argTys retTy) <$> f (head ns)
+#endif
+
+instance HasName Foreign where
+  name f (ImportF cc saf str n ty) =
+    (\n' -> ImportF cc saf str n' ty) <$> f n
+  name f (ExportF cc str n ty) =
+    (\n' -> ExportF cc str n' ty) <$> f n
+
+#if MIN_VERSION_template_haskell(2,8,0)
+instance HasName RuleBndr where
+  name f (RuleVar n) = RuleVar <$> f n
+  name f (TypedRuleVar n ty) = (`TypedRuleVar` ty) <$> f n
+#endif
+
+#if MIN_VERSION_template_haskell(2,11,0)
+instance HasName TypeFamilyHead where
+  name f (TypeFamilyHead n tvbs frs mia) =
+    (\n' -> TypeFamilyHead n' tvbs frs mia) <$> f n
+
+instance HasName InjectivityAnn where
+  name f (InjectivityAnn n deps) = (`InjectivityAnn` deps) <$> f n
+#endif
 
 -- | Contains some amount of `Type`s inside
 class HasTypes t where
@@ -336,6 +427,22 @@ instance HasTypes Con where
   types f (InfixC t1 n t2) = InfixC <$> _2 (types f) t1
                                        <*> pure n <*> _2 (types f) t2
   types f (ForallC vb ctx con)    = ForallC vb ctx <$> types f con
+#if MIN_VERSION_template_haskell(2,11,0)
+  types f (GadtC ns argTys retTy) =
+    GadtC    ns <$> traverse (_2 (types f)) argTys <*> types f retTy
+  types f (RecGadtC ns argTys retTy) =
+    RecGadtC ns <$> traverse (_3 (types f)) argTys <*> types f retTy
+#endif
+
+instance HasTypes Foreign where
+  types f (ImportF cc saf str n t) = ImportF cc saf str n <$> types f t
+  types f (ExportF cc     str n t) = ExportF cc     str n <$> types f t
+
+#if MIN_VERSION_template_haskell(2,9,0)
+instance HasTypes TySynEqn where
+  types f (TySynEqn lhss rhs) = TySynEqn <$> traverse (types f) lhss
+                                         <*> types f rhs
+#endif
 
 instance HasTypes t => HasTypes [t] where
   types = traverse . types
@@ -363,6 +470,15 @@ instance HasTypeVars Type where
   typeVarsEx s f (SigT t k)          = (`SigT` k) <$> typeVarsEx s f t
   typeVarsEx s f (ForallT bs ctx ty) = ForallT bs <$> typeVarsEx s' f ctx <*> typeVarsEx s' f ty
        where s' = s `Set.union` setOf typeVars bs
+#if MIN_VERSION_template_haskell(2,11,0)
+  typeVarsEx s f (InfixT  t1 n t2)   = InfixT  <$> typeVarsEx s f t1
+                                               <*> pure n
+                                               <*> typeVarsEx s f t2
+  typeVarsEx s f (UInfixT t1 n t2)   = UInfixT <$> typeVarsEx s f t1
+                                               <*> pure n
+                                               <*> typeVarsEx s f t2
+  typeVarsEx s f (ParensT t)         = ParensT <$> typeVarsEx s f t
+#endif
   typeVarsEx _ _ t                   = pure t
 
 #if !MIN_VERSION_template_haskell(2,10,0)
@@ -378,6 +494,14 @@ instance HasTypeVars Con where
        where g (i, t) = (,) i <$> typeVarsEx s f t
   typeVarsEx s f (ForallC bs ctx c) = ForallC bs <$> typeVarsEx s' f ctx <*> typeVarsEx s' f c
        where s' = s `Set.union` setOf typeVars bs
+#if MIN_VERSION_template_haskell(2,11,0)
+  typeVarsEx s f (GadtC ns argTys retTy) =
+    GadtC ns <$> traverseOf (traverse . _2) (typeVarsEx s f) argTys
+             <*> typeVarsEx s f retTy
+  typeVarsEx s f (RecGadtC ns argTys retTy) =
+    RecGadtC ns <$> traverseOf (traverse . _3) (typeVarsEx s f) argTys
+                <*> typeVarsEx s f retTy
+#endif
 
 instance HasTypeVars t => HasTypeVars [t] where
   typeVarsEx s = traverse . typeVarsEx s
@@ -404,6 +528,11 @@ instance SubstType Type where
     where m' = foldrOf typeVars Map.delete m bs
   substType m (SigT t k)          = SigT (substType m t) k
   substType m (AppT l r)          = AppT (substType m l) (substType m r)
+#if MIN_VERSION_template_haskell(2,11,0)
+  substType m (InfixT  t1 n t2)   = InfixT  (substType m t1) n (substType m t2)
+  substType m (UInfixT t1 n t2)   = UInfixT (substType m t1) n (substType m t2)
+  substType m (ParensT t)         = ParensT (substType m t)
+#endif
   substType _ t                   = t
 
 instance SubstType t => SubstType [t] where
@@ -416,17 +545,43 @@ instance SubstType Pred where
 #endif
 
 -- | Provides a 'Traversal' of the types of each field of a constructor.
-conFields :: Traversal' Con StrictType
+conFields :: Traversal' Con
+#if MIN_VERSION_template_haskell(2,11,0)
+                            BangType
+#else
+                            StrictType
+#endif
 conFields f (NormalC n fs)      = NormalC n <$> traverse f fs
-conFields f (RecC n fs)         = RecC n <$> traverse sans_var fs
-  where sans_var (fn,s,t) = (\(s', t') -> (fn,s',t')) <$> f (s, t)
+conFields f (RecC n fs)         = RecC n <$> traverse (sansVar f) fs
 conFields f (InfixC l n r)      = InfixC <$> f l <*> pure n <*> f r
 conFields f (ForallC bds ctx c) = ForallC bds ctx <$> conFields f c
+#if MIN_VERSION_template_haskell(2,11,0)
+conFields f (GadtC ns argTys retTy) =
+  GadtC ns <$> traverse f argTys <*> pure retTy
+conFields f (RecGadtC ns argTys retTy) =
+  RecGadtC ns <$> traverse (sansVar f) argTys <*> pure retTy
+#endif
+
+#if MIN_VERSION_template_haskell(2,11,0)
+sansVar :: Traversal' VarBangType   BangType
+#else
+sansVar :: Traversal' VarStrictType StrictType
+#endif
+sansVar f (fn,s,t) = (\(s', t') -> (fn,s',t')) <$> f (s, t)
 
 -- | 'Traversal' of the types of the /named/ fields of a constructor.
-conNamedFields :: Traversal' Con VarStrictType
+conNamedFields :: Traversal' Con
+#if MIN_VERSION_template_haskell(2,11,0)
+                                 VarBangType
+#else
+                                 VarStrictType
+#endif
 conNamedFields f (RecC n fs) = RecC n <$> traverse f fs
 conNamedFields f (ForallC a b fs) = ForallC a b <$> conNamedFields f fs
+#if MIN_VERSION_template_haskell(2,11,0)
+conNamedFields f (RecGadtC ns argTys retTy) =
+  RecGadtC ns <$> traverse f argTys <*> pure retTy
+#endif
 conNamedFields _ c = pure c
 
 -- Lenses and Prisms
@@ -512,8 +667,53 @@ clauseDecs = lens g s where
    g (Clause _ _ ds) = ds
    s (Clause x y _ ) = Clause x y
 
+#if MIN_VERSION_template_haskell(2,11,0)
+injectivityAnnOutput :: Lens' InjectivityAnn Name
+injectivityAnnOutput = lens g s where
+   g (InjectivityAnn o _)   = o
+   s (InjectivityAnn _ i) o = InjectivityAnn o i
+
+injectivityAnnInputs :: Lens' InjectivityAnn [Name]
+injectivityAnnInputs = lens g s where
+   g (InjectivityAnn _ i) = i
+   s (InjectivityAnn o _) = InjectivityAnn o
+
+typeFamilyHeadName :: Lens' TypeFamilyHead Name
+typeFamilyHeadName = lens g s where
+  g (TypeFamilyHead n _    _  _ )   = n
+  s (TypeFamilyHead _ tvbs rs ia) n = TypeFamilyHead n tvbs rs ia
+
+typeFamilyHeadTyVarBndrs :: Lens' TypeFamilyHead [TyVarBndr]
+typeFamilyHeadTyVarBndrs = lens g s where
+  g (TypeFamilyHead _ tvbs _  _ )      = tvbs
+  s (TypeFamilyHead n _    rs ia) tvbs = TypeFamilyHead n tvbs rs ia
+
+typeFamilyHeadResultSig :: Lens' TypeFamilyHead FamilyResultSig
+typeFamilyHeadResultSig = lens g s where
+  g (TypeFamilyHead _ _    rs _ )    = rs
+  s (TypeFamilyHead n tvbs _  ia) rs = TypeFamilyHead n tvbs rs ia
+
+typeFamilyHeadInjectivityAnn :: Lens' TypeFamilyHead (Maybe InjectivityAnn)
+typeFamilyHeadInjectivityAnn = lens g s where
+  g (TypeFamilyHead _ _    _  ia) = ia
+  s (TypeFamilyHead n tvbs rs _ ) = TypeFamilyHead n tvbs rs
+
+bangSourceUnpackedness :: Lens' Bang SourceUnpackedness
+bangSourceUnpackedness = lens g s where
+  g (Bang su _ )    = su
+  s (Bang _  ss) su = Bang su ss
+
+bangSourceStrictness :: Lens' Bang SourceStrictness
+bangSourceStrictness = lens g s where
+  g (Bang _  su) = su
+  s (Bang ss _ ) = Bang ss
+#endif
+
 #if MIN_VERSION_template_haskell(2,8,0)
 _ClassI :: Prism' Info (Dec, [InstanceDec])
+#else
+_ClassI :: Prism' Info (Dec, [Dec])
+#endif
 _ClassI
   = prism' reviewer remitter
   where
@@ -521,24 +721,20 @@ _ClassI
       remitter (ClassI x y) = Just (x, y)
       remitter _ = Nothing
 
-_ClassOpI :: Prism' Info (Name, Type, ParentName, Fixity)
+#if MIN_VERSION_template_haskell(2,11,0)
+_ClassOpI :: Prism' Info (Name, Type, ParentName)
 _ClassOpI
   = prism' reviewer remitter
   where
-      reviewer (x, y, z, w) = ClassOpI x y z w
-      remitter (ClassOpI x y z w) = Just (x, y, z, w)
+      reviewer (x, y, z) = ClassOpI x y z
+      remitter (ClassOpI x y z) = Just (x, y, z)
       remitter _ = Nothing
-
 #else
-_ClassI :: Prism' Info (Dec, [Dec])
-_ClassI
-  = prism' reviewer remitter
-  where
-      reviewer (x, y) = ClassI x y
-      remitter (ClassI x y) = Just (x, y)
-      remitter _ = Nothing
-
+# if MIN_VERSION_template_haskell(2,8,0)
+_ClassOpI :: Prism' Info (Name, Type, ParentName, Fixity)
+# else
 _ClassOpI :: Prism' Info (Name, Type, Name, Fixity)
+# endif
 _ClassOpI
   = prism' reviewer remitter
   where
@@ -555,32 +751,11 @@ _TyConI
       remitter (TyConI x) = Just x
       remitter _ = Nothing
 
-#if MIN_VERSION_template_haskell(2,8,0)
+#if MIN_VERSION_template_haskell(2,11,0)
 _FamilyI :: Prism' Info (Dec, [InstanceDec])
-_FamilyI
-  = prism' reviewer remitter
-  where
-      reviewer (x, y) = FamilyI x y
-      remitter (FamilyI x y) = Just (x, y)
-      remitter _ = Nothing
-
-_PrimTyConI :: Prism' Info (Name, Arity, Unlifted)
-_PrimTyConI
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z) = PrimTyConI x y z
-      remitter (PrimTyConI x y z) = Just (x, y, z)
-      remitter _ = Nothing
-
-_DataConI :: Prism' Info (Name, Type, ParentName, Fixity)
-_DataConI
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z, w) = DataConI x y z w
-      remitter (DataConI x y z w) = Just (x, y, z, w)
-      remitter _ = Nothing
 #else
 _FamilyI :: Prism' Info (Dec, [Dec])
+#endif
 _FamilyI
   = prism' reviewer remitter
   where
@@ -588,7 +763,11 @@ _FamilyI
       remitter (FamilyI x y) = Just (x, y)
       remitter _ = Nothing
 
+#if MIN_VERSION_template_haskell(2,8,0)
+_PrimTyConI :: Prism' Info (Name, Arity, Unlifted)
+#else
 _PrimTyConI :: Prism' Info (Name, Int, Bool)
+#endif
 _PrimTyConI
   = prism' reviewer remitter
   where
@@ -596,16 +775,37 @@ _PrimTyConI
       remitter (PrimTyConI x y z) = Just (x, y, z)
       remitter _ = Nothing
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_DataConI :: Prism' Info (Name, Type, ParentName)
+_DataConI
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = DataConI x y z
+      remitter (DataConI x y z) = Just (x, y, z)
+      remitter _ = Nothing
+#else
+# if MIN_VERSION_template_haskell(2,8,0)
+_DataConI :: Prism' Info (Name, Type, ParentName, Fixity)
+# else
 _DataConI :: Prism' Info (Name, Type, Name, Fixity)
+# endif
 _DataConI
   = prism' reviewer remitter
   where
       reviewer (x, y, z, w) = DataConI x y z w
       remitter (DataConI x y z w) = Just (x, y, z, w)
       remitter _ = Nothing
-
 #endif
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_VarI :: Prism' Info (Name, Type, Maybe Dec)
+_VarI
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = VarI x y z
+      remitter (VarI x y z) = Just (x, y, z)
+      remitter _ = Nothing
+#else
 _VarI :: Prism' Info (Name, Type, Maybe Dec, Fixity)
 _VarI
   = prism' reviewer remitter
@@ -613,6 +813,7 @@ _VarI
       reviewer (x, y, z, w) = VarI x y z w
       remitter (VarI x y z w) = Just (x, y, z, w)
       remitter _ = Nothing
+#endif
 
 _TyVarI :: Prism' Info (Name, Type)
 _TyVarI
@@ -638,22 +839,6 @@ _ValD
       remitter (ValD x y z) = Just (x, y, z)
       remitter _ = Nothing
 
-_DataD :: Prism' Dec (Cxt, Name, [TyVarBndr], [Con], [Name])
-_DataD
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z, w, u) = DataD x y z w u
-      remitter (DataD x y z w u) = Just (x, y, z, w, u)
-      remitter _ = Nothing
-
-_NewtypeD :: Prism' Dec (Cxt, Name, [TyVarBndr], Con, [Name])
-_NewtypeD
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z, w, u) = NewtypeD x y z w u
-      remitter (NewtypeD x y z w u) = Just (x, y, z, w, u)
-      remitter _ = Nothing
-
 _TySynD :: Prism' Dec (Name, [TyVarBndr], Type)
 _TySynD
   = prism' reviewer remitter
@@ -670,14 +855,53 @@ _ClassD
       remitter (ClassD x y z w u) = Just (x, y, z, w, u)
       remitter _ = Nothing
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_InstanceD :: Prism' Dec (Maybe Overlap, Cxt, Type, [Dec])
+#else
 _InstanceD :: Prism' Dec (Cxt, Type, [Dec])
+#endif
 _InstanceD
   = prism' reviewer remitter
   where
+#if MIN_VERSION_template_haskell(2,11,0)
+      reviewer (x, y, z, w) = InstanceD x y z w
+      remitter (InstanceD x y z w) = Just (x, y, z, w)
+#else
       reviewer (x, y, z) = InstanceD x y z
-      remitter (InstanceD x y z) = Just (x, y, z)
+      remitter (InstanceD x y z) = Just ( x, y, z)
+#endif
       remitter _ = Nothing
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_Overlappable  :: Prism' Overlap  ()
+_Overlappable  = prism' reviewer remitter
+  where
+      reviewer () = Overlappable
+      remitter Overlappable = Just  ()
+      remitter _ = Nothing
+
+_Overlapping :: Prism' Overlap ()
+_Overlapping = prism' reviewer remitter
+  where
+      reviewer () = Overlapping
+      remitter Overlapping = Just ()
+      remitter _ = Nothing
+
+_Overlaps ::  Prism' Overlap  ()
+_Overlaps =  prism' reviewer remitter
+  where
+      reviewer () =  Overlaps
+      remitter Overlaps = Just ()
+      remitter _ = Nothing
+
+_Incoherent  :: Prism' Overlap ()
+_Incoherent  = prism' reviewer remitter
+  where
+      reviewer () = Incoherent
+      remitter Incoherent = Just ()
+      remitter _ = Nothing
+#endif
+
 _SigD :: Prism' Dec (Name, Type)
 _SigD
   = prism' reviewer remitter
@@ -712,30 +936,6 @@ _PragmaD
       remitter (PragmaD x) = Just x
       remitter _ = Nothing
 
-_FamilyD :: Prism' Dec (FamFlavour, Name, [TyVarBndr], Maybe Kind)
-_FamilyD
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z, w) = FamilyD x y z w
-      remitter (FamilyD x y z w) = Just (x, y, z, w)
-      remitter _ = Nothing
-
-_DataInstD :: Prism' Dec (Cxt, Name, [Type], [Con], [Name])
-_DataInstD
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z, w, u) = DataInstD x y z w u
-      remitter (DataInstD x y z w u) = Just (x, y, z, w, u)
-      remitter _ = Nothing
-
-_NewtypeInstD :: Prism' Dec (Cxt, Name, [Type], Con, [Name])
-_NewtypeInstD
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z, w, u) = NewtypeInstD x y z w u
-      remitter (NewtypeInstD x y z w u) = Just (x, y, z, w, u)
-      remitter _ = Nothing
-
 #if MIN_VERSION_template_haskell(2,9,0)
 _TySynInstD :: Prism' Dec (Name, TySynEqn)
 _TySynInstD
@@ -745,14 +945,6 @@ _TySynInstD
       remitter (TySynInstD x y) = Just (x, y)
       remitter _ = Nothing
 
-_ClosedTypeFamilyD :: Prism' Dec (Name, [TyVarBndr], Maybe Kind, [TySynEqn])
-_ClosedTypeFamilyD
-  = prism' reviewer remitter
-  where
-      reviewer (x, y, z, w) = ClosedTypeFamilyD x y z w
-      remitter (ClosedTypeFamilyD x y z w) = Just (x, y, z, w)
-      remitter _ = Nothing
-
 _RoleAnnotD :: Prism' Dec (Name, [Role])
 _RoleAnnotD
   = prism' reviewer remitter
@@ -789,8 +981,122 @@ _DefaultSigD
       remitter _ = Nothing
 #endif
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_ClosedTypeFamilyD :: Prism' Dec (TypeFamilyHead, [TySynEqn])
+_ClosedTypeFamilyD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y) = ClosedTypeFamilyD x y
+      remitter (ClosedTypeFamilyD x y) = Just (x, y)
+      remitter _ = Nothing
+#elif MIN_VERSION_template_haskell(2,9,0)
+_ClosedTypeFamilyD :: Prism' Dec (Name, [TyVarBndr], Maybe Kind, [TySynEqn])
+_ClosedTypeFamilyD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w) = ClosedTypeFamilyD x y z w
+      remitter (ClosedTypeFamilyD x y z w) = Just (x, y, z, w)
+      remitter _ = Nothing
+#endif
+
+#if MIN_VERSION_template_haskell(2,11,0)
+_DataD :: Prism' Dec (Cxt, Name, [TyVarBndr], Maybe Kind, [Con], Cxt)
+_DataD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u, v) = DataD x y z w u v
+      remitter (DataD x y z w u v) = Just (x, y, z, w, u, v)
+      remitter _ = Nothing
+
+_NewtypeD :: Prism' Dec (Cxt, Name, [TyVarBndr], Maybe Kind, Con, Cxt)
+_NewtypeD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u, v) = NewtypeD x y z w u v
+      remitter (NewtypeD x y z w u v) = Just (x, y, z, w, u, v)
+      remitter _ = Nothing
+
+_DataInstD :: Prism' Dec (Cxt, Name, [Type], Maybe Kind, [Con], Cxt)
+_DataInstD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u, v) = DataInstD x y z w u v
+      remitter (DataInstD x y z w u v) = Just (x, y, z, w, u, v)
+      remitter _ = Nothing
+
+_NewtypeInstD :: Prism' Dec (Cxt, Name, [Type], Maybe Kind, Con, Cxt)
+_NewtypeInstD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u, v) = NewtypeInstD x y z w u v
+      remitter (NewtypeInstD x y z w u v) = Just (x, y, z, w, u, v)
+      remitter _ = Nothing
+
+_DataFamilyD :: Prism' Dec (Name, [TyVarBndr], Maybe Kind)
+_DataFamilyD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = DataFamilyD x y z
+      remitter (DataFamilyD x y z) = Just (x, y, z)
+      remitter _ = Nothing
+
+_OpenTypeFamilyD :: Prism' Dec TypeFamilyHead
+_OpenTypeFamilyD
+  = prism' reviewer remitter
+  where
+      reviewer = OpenTypeFamilyD
+      remitter (OpenTypeFamilyD x) = Just x
+      remitter _ = Nothing
+#else
+_DataD :: Prism' Dec (Cxt, Name, [TyVarBndr], [Con], [Name])
+_DataD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u) = DataD x y z w u
+      remitter (DataD x y z w u) = Just (x, y, z, w, u)
+      remitter _ = Nothing
+
+_NewtypeD :: Prism' Dec (Cxt, Name, [TyVarBndr], Con, [Name])
+_NewtypeD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u) = NewtypeD x y z w u
+      remitter (NewtypeD x y z w u) = Just (x, y, z, w, u)
+      remitter _ = Nothing
+
+_DataInstD :: Prism' Dec (Cxt, Name, [Type], [Con], [Name])
+_DataInstD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u) = DataInstD x y z w u
+      remitter (DataInstD x y z w u) = Just (x, y, z, w, u)
+      remitter _ = Nothing
+
+_NewtypeInstD :: Prism' Dec (Cxt, Name, [Type], Con, [Name])
+_NewtypeInstD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w, u) = NewtypeInstD x y z w u
+      remitter (NewtypeInstD x y z w u) = Just (x, y, z, w, u)
+      remitter _ = Nothing
+
+_FamilyD :: Prism' Dec (FamFlavour, Name, [TyVarBndr], Maybe Kind)
+_FamilyD
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z, w) = FamilyD x y z w
+      remitter (FamilyD x y z w) = Just (x, y, z, w)
+      remitter _ = Nothing
+#endif
+
 _NormalC ::
-  Prism' Con (Name, [StrictType])
+  Prism' Con ( Name
+#if MIN_VERSION_template_haskell(2,11,0)
+             , [BangType]
+#else
+             , [StrictType]
+#endif
+             )
 _NormalC
   = prism' reviewer remitter
   where
@@ -799,7 +1105,13 @@ _NormalC
       remitter _ = Nothing
 
 _RecC ::
-  Prism' Con (Name, [VarStrictType])
+  Prism' Con ( Name
+#if MIN_VERSION_template_haskell(2,11,0)
+             , [VarBangType]
+#else
+             , [VarStrictType]
+#endif
+             )
 _RecC
   = prism' reviewer remitter
   where
@@ -808,9 +1120,12 @@ _RecC
       remitter _ = Nothing
 
 _InfixC ::
-  Prism' Con (StrictType,
-              Name,
-              StrictType)
+  Prism' Con
+#if MIN_VERSION_template_haskell(2,11,0)
+             (BangType,   Name, BangType  )
+#else
+             (StrictType, Name, StrictType)
+#endif
 _InfixC
   = prism' reviewer remitter
   where
@@ -826,6 +1141,97 @@ _ForallC
       remitter (ForallC x y z) = Just (x, y, z)
       remitter _ = Nothing
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_GadtC :: Prism' Con ([Name], [BangType], Type)
+_GadtC
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = GadtC x y z
+      remitter (GadtC x y z) = Just (x, y, z)
+      remitter _ = Nothing
+
+_RecGadtC :: Prism' Con ([Name], [VarBangType], Type)
+_RecGadtC
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = RecGadtC x y z
+      remitter (RecGadtC x y z) = Just (x, y, z)
+      remitter _ = Nothing
+#endif
+
+#if MIN_VERSION_template_haskell(2,11,0)
+_NoSourceUnpackedness :: Prism' SourceUnpackedness ()
+_NoSourceUnpackedness
+  = prism' reviewer remitter
+  where
+      reviewer () = NoSourceUnpackedness
+      remitter NoSourceUnpackedness = Just ()
+      remitter _ = Nothing
+
+_SourceNoUnpack :: Prism' SourceUnpackedness ()
+_SourceNoUnpack
+  = prism' reviewer remitter
+  where
+      reviewer () = SourceNoUnpack
+      remitter SourceNoUnpack = Just ()
+      remitter _ = Nothing
+
+_SourceUnpack :: Prism' SourceUnpackedness ()
+_SourceUnpack
+  = prism' reviewer remitter
+  where
+      reviewer () = SourceUnpack
+      remitter SourceUnpack = Just ()
+      remitter _ = Nothing
+
+_NoSourceStrictness :: Prism' SourceStrictness ()
+_NoSourceStrictness
+  = prism' reviewer remitter
+  where
+      reviewer () = NoSourceStrictness
+      remitter NoSourceStrictness = Just ()
+      remitter _ = Nothing
+
+_SourceLazy :: Prism' SourceStrictness ()
+_SourceLazy
+  = prism' reviewer remitter
+  where
+      reviewer () = SourceLazy
+      remitter SourceLazy = Just ()
+      remitter _ = Nothing
+
+_SourceStrict :: Prism' SourceStrictness ()
+_SourceStrict
+  = prism' reviewer remitter
+  where
+      reviewer () = SourceStrict
+      remitter SourceStrict = Just ()
+      remitter _ = Nothing
+
+_DecidedLazy :: Prism' DecidedStrictness ()
+_DecidedLazy
+  = prism' reviewer remitter
+  where
+      reviewer () = DecidedLazy
+      remitter DecidedLazy = Just ()
+      remitter _ = Nothing
+
+_DecidedStrict :: Prism' DecidedStrictness ()
+_DecidedStrict
+  = prism' reviewer remitter
+  where
+      reviewer () = DecidedStrict
+      remitter DecidedStrict = Just ()
+      remitter _ = Nothing
+
+_DecidedUnpack :: Prism' DecidedStrictness ()
+_DecidedUnpack
+  = prism' reviewer remitter
+  where
+      reviewer () = DecidedUnpack
+      remitter DecidedUnpack = Just ()
+      remitter _ = Nothing
+#else
 _IsStrict :: Prism' Strict ()
 _IsStrict
   = prism' reviewer remitter
@@ -849,6 +1255,7 @@ _Unpacked
       reviewer () = Unpacked
       remitter Unpacked = Just ()
       remitter _ = Nothing
+#endif
 
 _ImportF :: Prism' Foreign (Callconv, Safety, String, Name, Type)
 _ImportF
@@ -1361,6 +1768,16 @@ _StaticE
       remitter _ = Nothing
 #endif
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_UnboundVarE :: Prism' Exp Name
+_UnboundVarE
+  = prism' reviewer remitter
+  where
+      reviewer = UnboundVarE
+      remitter (UnboundVarE x) = Just x
+      remitter _ = Nothing
+#endif
+
 _GuardedB :: Prism' Body [(Guard, Exp)]
 _GuardedB
   = prism' reviewer remitter
@@ -1539,6 +1956,16 @@ _StringPrimL
       remitter _ = Nothing
 #endif
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_CharPrimL :: Prism' Lit Char
+_CharPrimL
+  = prism' reviewer remitter
+  where
+      reviewer = CharPrimL
+      remitter (CharPrimL x) = Just x
+      remitter _ = Nothing
+#endif
+
 _LitP :: Prism' Pat Lit
 _LitP
   = prism' reviewer remitter
@@ -1809,6 +2236,40 @@ _LitT
       remitter _ = Nothing
 #endif
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_InfixT :: Prism' Type (Type, Name, Type)
+_InfixT
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = InfixT x y z
+      remitter (InfixT x y z) = Just (x, y, z)
+      remitter _ = Nothing
+
+_UInfixT :: Prism' Type (Type, Name, Type)
+_UInfixT
+  = prism' reviewer remitter
+  where
+      reviewer (x, y, z) = UInfixT x y z
+      remitter (UInfixT x y z) = Just (x, y, z)
+      remitter _ = Nothing
+
+_ParensT :: Prism' Type Type
+_ParensT
+  = prism' reviewer remitter
+  where
+      reviewer = ParensT
+      remitter (ParensT x) = Just x
+      remitter _ = Nothing
+
+_WildCardT :: Prism' Type ()
+_WildCardT
+  = prism' reviewer remitter
+  where
+      reviewer () = WildCardT
+      remitter WildCardT = Just ()
+      remitter _ = Nothing
+#endif
+
 _PlainTV :: Prism' TyVarBndr Name
 _PlainTV
   = prism' reviewer remitter
@@ -1825,6 +2286,32 @@ _KindedTV
       remitter (KindedTV x y) = Just (x, y)
       remitter _ = Nothing
 
+#if MIN_VERSION_template_haskell(2,11,0)
+_NoSig :: Prism' FamilyResultSig ()
+_NoSig
+  = prism' reviewer remitter
+  where
+      reviewer () = NoSig
+      remitter NoSig = Just ()
+      remitter _ = Nothing
+
+_KindSig :: Prism' FamilyResultSig Kind
+_KindSig
+  = prism' reviewer remitter
+  where
+      reviewer = KindSig
+      remitter (KindSig x) = Just x
+      remitter _ = Nothing
+
+_TyVarSig :: Prism' FamilyResultSig TyVarBndr
+_TyVarSig
+  = prism' reviewer remitter
+  where
+      reviewer = TyVarSig
+      remitter (TyVarSig x) = Just x
+      remitter _ = Nothing
+#endif
+
 #if MIN_VERSION_template_haskell(2,8,0)
 _NumTyLit :: Prism' TyLit Integer
 _NumTyLit
