@@ -1,9 +1,17 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE Rank2Types #-}
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE ExtendedDefaultRules #-}
 {-# LANGUAGE LiberalTypeSynonyms #-}
 {-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE KindSignatures #-}
+#if __GLASGOW_HASKELL__ >= 706
+{-# LANGUAGE GADTs #-}
+{-# LANGUAGE PolyKinds #-}
+{-# LANGUAGE TypeOperators #-}
+{-# LANGUAGE ConstraintKinds #-}
+#endif
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Main (properties)
@@ -29,6 +37,7 @@ import Data.Text.Strict.Lens
 import Data.Maybe
 import Data.List.Lens
 import Data.Functor.Compose
+import GHC.Exts (Constraint)
 import Numeric (showHex, showOct, showSigned)
 import Numeric.Lens
 import Control.Lens.Properties (isIso, isLens, isPrism, isSetter, isTraversal)
@@ -96,5 +105,37 @@ prop_base_readFail (s :: String) =
     sPos = case s of { ('-':s') -> s'; _ -> s }
     isValidChar c = isAscii c && isAlphaNum c
 
+-- Things that should typecheck but that we don't need to run
+#if __GLASGOW_HASKELL__ >= 708
+data Foo (a :: Constraint) (b :: Constraint) where
+  Foo :: Foo (Num Int) b
+
+sampleExtremePoly :: Equality s t a b -> Foo a (Functor b) -> Foo s (Functor t)
+sampleExtremePoly f foo = f foo
+#endif
+
+#if __GLASGOW_HASKELL__ >= 706
+samplePolyEquality :: Equality Monad Identity Monad Identity
+samplePolyEquality f = f
+
+lessSimplePoly :: forall (s :: k1) (t :: k2) (a :: k1) (b :: k2) .
+                  Equality a b a b
+lessSimplePoly f = f
+
+equalityAnEqualityPoly ::
+       forall (s :: k1) (t :: k2) (a :: k1) (b :: k2) .
+       Equality s t a b -> AnEquality s t a b
+equalityAnEqualityPoly f = f
+#else
+lessSimple :: Equality a b a b
+lessSimple f = f
+
+equalityAnEquality :: Equality s t a b -> AnEquality s t a b
+equalityAnEquality f = f
+#endif
+equalityIso :: Equality s t a b -> Iso s t a b
+equalityIso f = f
+
+
 main :: IO ()
 main = $defaultMainGenerator
